/*---------------------------------------------------------------------------*/
/* Program:  color_histo.c                                                   */
/*                                                                           */
/* Purpose:  This program calculates a histogram of color values in an       */
/*           image.  At present a 6x6x6 histogram is used.                   */
/*                                                                           */
/* Author:   John Gauch                                                      */
/*                                                                           */
/* Date:     May 1, 1997                                                     */
/*                                                                           */
/* Note:     Copyright (C) The University of Kansas, 1997                    */
/*---------------------------------------------------------------------------*/
#include <IM.h>

/*---------------------------------------------------------------------------*/
/* Purpose:  This is the main program.                                       */
/*---------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
   /* Image variables */
   char Name1[50] = "";
   char Name2[50] = "";
   IM_TYPE *Image1;
   COLOR_TYPE *Data1;
   float Histo1[6][6][6];
   float Histo2[6][6][6];
   int PixType, Xdim, Ydim, Zdim, DimCnt;

   /* Program variables */
   int Debug = FALSE;
   int i = 0;
   int Count;
   int r, g, b;
   float diff, total;

   /* Interpret program options */
   fprintf(stderr, "COLOR_HISTO Program - KUIM Version 1.0\n\n");
   while ((++i < argc) && (argv[i][0] == '-'))
      switch (argv[i][1])
      {
      case 'd':
	 Debug = TRUE;
	 break;
      default:
	 Error("Invalid option encountered");
	 break;
      }

   /* Check number of file names */
   if (argc - i < 1)
   {
      fprintf(stderr, "Usage: color_histo [options] infile [infile]\n");
      fprintf(stderr, "       [-d]    Print debugging information\n");
      exit(1);
   }

   /* Get image file names from argument list */
   if ((argc > i) && (sscanf(argv[i++], "%s", Name1) == 0))
      Error("Could not get input file name");
   if ((argc > i) && (sscanf(argv[i++], "%s", Name2) == 0))
      Error("Could not get input file name");

   /* Read input image */
   Image1 = im_open(Name1, &PixType, &Xdim, &Ydim, &Zdim, &DimCnt);
   Data1 = (COLOR_TYPE *) im_alloc1D(Image1, COLOR);
   im_read(Image1, COLOR, (char *) Data1);

   /* Initialize color histogram */
   for (r = 0; r < 6; r++)
      for (g = 0; g < 6; g++)
	 for (b = 0; b < 6; b++)
	    Histo1[r][g][b] = 0;

   /* Get intensity histogram */
   Count = Xdim * Ydim * Zdim;
   for (i = 0; i < Count; i++)
   {
      r = Data1[i].r / 43;
      g = Data1[i].g / 43;
      b = Data1[i].b / 43;
      Histo1[r][g][b] += (float) (1.0 / Count);
   }

   /* Print histogram */
   if (strcmp(Name2, "") == 0)
   {
      for (r = 0; r < 6; r++)
      {
	 for (g = 0; g < 6; g++)
	 {
	    for (b = 0; b < 6; b++)
	       printf("%5.2f", Histo1[r][g][b]);
	    printf("\n");
	 }
	 printf("\n");
      }
      exit(0);
   }

   /* Read second input image */
   Image1 = im_open(Name2, &PixType, &Xdim, &Ydim, &Zdim, &DimCnt);
   Data1 = (COLOR_TYPE *) im_alloc1D(Image1, COLOR);
   im_read(Image1, COLOR, (char *) Data1);

   /* Initialize color histogram */
   for (r = 0; r < 6; r++)
      for (g = 0; g < 6; g++)
	 for (b = 0; b < 6; b++)
	    Histo2[r][g][b] = 0;

   /* Get intensity histogram */
   Count = Xdim * Ydim * Zdim;
   for (i = 0; i < Count; i++)
   {
      r = Data1[i].r / 43;
      g = Data1[i].g / 43;
      b = Data1[i].b / 43;
      Histo2[r][g][b] += (float) (1.0 / Count);
   }

   /* Print difference histogram */
   total = 0;
   for (r = 0; r < 6; r++)
   {
      for (g = 0; g < 6; g++)
      {
	 for (b = 0; b < 6; b++)
	 {
	    diff = (float) fabs((double) (Histo1[r][g][b] - Histo2[r][g][b]));
	    total += diff;
	    printf("%5.2f", diff);
	 }
	 printf("\n");
      }
      printf("\n");
   }
   printf("total difference = %5.2f\n", total);

   return (0);
}
