/*---------------------------------------------------------------------------*/
/*                                                                           */
/* Program:  mandel.c                                                        */
/*                                                                           */
/* Purpose:  This program calculates the Mandelbroit set for a given         */
/*           window of the complex plane.                                    */
/*                                                                           */
/* Author:   John Gauch                                                      */
/*                                                                           */
/* Date:     December 2, 1996                                                */
/*                                                                           */
/* Note:     Copyright (C) The University of Kansas, 1996                    */
/*---------------------------------------------------------------------------*/
#include <IM.h>

/*---------------------------------------------------------------------------*/
/*                                                                           */
/* Purpose:  This is the main program.                                       */
/*                                                                           */
/*---------------------------------------------------------------------------*/
main(int argc, char *argv[])
{
   /* Image variables */
   char Name1[50];
   IM_TYPE *Image1;
   SHORT_TYPE **Data1;
   int Xdim, Ydim, Zdim;

   /* Program variables */
   float Cre, Cim, Zre, Zim, length, temp;
   float Lre = -1.6, Lim = -1.2, Hre = 0.8, Him = 1.2;
   int count;
   int Debug = FALSE;
   int i = 0, x, y;

   /* Interpret program options */
   printf("MANDEL Program - KUIM Version 1.0\n\n");
   while ((++i < argc) && (argv[i][0] == '-'))
      switch (argv[i][1])
      {
      case 'w':
	 if (sscanf(argv[++i], "%f", &Lre) == 0)
	    Error("Could not get float argument");
	 if (sscanf(argv[++i], "%f", &Lim) == 0)
	    Error("Could not get float argument");
	 if (sscanf(argv[++i], "%f", &Hre) == 0)
	    Error("Could not get float argument");
	 if (sscanf(argv[++i], "%f", &Him) == 0)
	    Error("Could not get float argument");
	 break;
      case 'd':
	 Debug = TRUE;
	 break;
      default:
	 Error("Invalid option encountered");
	 break;
      }

   /* Check number of file names */
   if (argc - i != 1)
   {
      fprintf(stderr, "Usage: mandel [options] outfile\n");
      fprintf(stderr, "       [-w # # # #] Window (-1.6 -1.2 0.8 1.2)\n");
      fprintf(stderr, "       [-d]  Print debugging information\n");
      return (1);
   }

   /* Get image file names from argument list */
   if (sscanf(argv[i++], "%s", Name1) == 0)
      Error("Could not get input file name");

   /* Create output image */
   Xdim = Ydim = 512;
   Zdim = 1;
   Image1 = im_create(Name1, SHORT, Xdim, Ydim, Zdim);
   Data1 = (SHORT_TYPE **) im_alloc2D(Image1, SHORT);

   /* Generate Mandebriot set */
   for (y = 0; y < Ydim; y++)
      for (x = 0; x < Xdim; x++)
      {
	 /* Initialize Z */
	 Zre = Cre = Lre + x * (Hre - Lre) / Xdim;
	 Zim = Cim = Lim + y * (Him - Lim) / Ydim;
	 length = Zre * Zre + Zim * Zim;

	 /* Iteratively update Z */
	 count = 0;
	 while ((length < 4) && (count < 100))
	 {
	    temp = Zre * Zre - Zim * Zim + Cre;
	    Zim = 2 * Zre * Zim + Cim;
	    Zre = temp;
	    length = Zre * Zre + Zim * Zim;
	    count++;
	 }

	 /* Set output pixel */
	 Data1[y][x] = count;
      }

   /* Write information to output image */
   im_write(Image1, SHORT, (char *) &(Data1[0][0]));
   im_free2D((char **) Data1);
   return (0);
}
