/*---------------------------------------------------------------------------*/
/* Program:  over.c                                                          */
/*                                                                           */
/* Purpose:  This program overlays nonzero pixels from one image on top      */
/*           of another image.  This is useful for showing boundaries        */
/*           of regions from image segmentation on the original image.       */
/*                                                                           */
/* Author:   John Gauch                                                      */
/*                                                                           */
/* Date:     March 22, 1995                                                  */
/*                                                                           */
/* Note:     Copyright (C) The University of Kansas, 1995                    */
/*---------------------------------------------------------------------------*/
#include <IM.h>

/*---------------------------------------------------------------------------*/
/* Purpose:  This is the main program.                                       */
/*---------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
   /* Image variables */
   char Name1[50];
   char Name2[50];
   char Name3[50];
   IM_TYPE *Image1;
   IM_TYPE *Image2;
   IM_TYPE *Image3;
   FLOAT_TYPE *Data1;
   FLOAT_TYPE *Data2;
   COLOR_TYPE *data2;
   int PixType, Xdim, Ydim, Zdim, DimCnt;
   int Xdim2, Ydim2, Zdim2;

   /* Program variables */
   int Debug = FALSE;
   int i = 0;
   int Count;
   float Value = 100.0;

   /* Interpret program options */
   printf("OVER Program - KUIM Version 2.0\n\n");
   while ((++i < argc) && (argv[i][0] == '-'))
      switch (argv[i][1])
      {
      case 'd':
	 Debug = TRUE;
	 break;
      case 'o':
	 if (sscanf(argv[++i], "%f", &Value) == 0)
	    Error("Could not get float argument");
	 break;
      default:
	 Error("Invalid option encountered");
	 break;
      }

   /* Check number of file names */
   if (argc - i != 3)
   {
      fprintf(stderr, "Usage: over [options] infile1 infile2 outfile\n");
      fprintf(stderr, "       [-d]    Print debugging information\n");
      fprintf(stderr, "       [-o #]  Overlay value (100)\n");
      exit(1);
   }

   /* Get image file names from argument list */
   if (sscanf(argv[i++], "%s", Name1) == 0)
      Error("Could not get input file name");
   if (sscanf(argv[i++], "%s", Name2) == 0)
      Error("Could not get input file name");
   if (sscanf(argv[i++], "%s", Name3) == 0)
      Error("Could not get output file name");

   /* Read input image1 */
   Image1 = im_open(Name1, &PixType, &Xdim, &Ydim, &Zdim, &DimCnt);
   Data1 = (FLOAT_TYPE *) im_alloc1D(Image1, FLOAT);
   im_read(Image1, FLOAT, (char *) Data1);

   /* Open input image2 */
   Image2 = im_open(Name2, &PixType, &Xdim2, &Ydim2, &Zdim2, &DimCnt);
   if ((Xdim != Xdim2) || (Ydim != Ydim2) || (Zdim != Zdim2))
      Error("Image dimensions do not match");

   /* Handle color images */
   if (PixType == COLOR || PixType == PSEUDO || PixType == JPEG_RGB)
   {
      /* Read input image1 */
      data2 = (COLOR_TYPE *) im_alloc1D(Image2, COLOR);
      im_read(Image2, COLOR, (char *) data2);

      /* Create output image */
      Image3 = im_create(Name3, PixType, Xdim, Ydim, Zdim);

      /* Overlay images */
      Count = Xdim * Ydim * Zdim;
      for (i = 0; i < Count; i++)
	 if (Data1[i] != 0)
	    data2[i].r = data2[i].g = data2[i].b = (BYTE_TYPE) Value;

      /* Write information to output image */
      im_write(Image3, COLOR, (char *) data2);
      im_free1D((char *) Data1);
      im_free1D((char *) data2);
   }

   /* Handle greyscale images */
   else
   {
      /* Read input image2 */
      Data2 = (FLOAT_TYPE *) im_alloc1D(Image2, FLOAT);
      im_read(Image2, FLOAT, (char *) Data2);

      /* Create output image */
      Image3 = im_create(Name3, PixType, Xdim, Ydim, Zdim);

      /* Overlay images */
      Count = Xdim * Ydim * Zdim;
      for (i = 0; i < Count; i++)
	 if (Data1[i] != 0)
	    Data2[i] = Value;

      /* Write information to output image */
      im_write(Image3, FLOAT, (char *) Data2);
      im_free1D((char *) Data1);
      im_free1D((char *) Data2);
   }
   return (0);
}
