/*---------------------------------------------------------------------------*/
/* Program:  plot_pts.c                                                      */
/*                                                                           */
/* Purpose:  This program plots a set of points on top of an image.          */
/*           These points are expected to be in an ascii file with           */
/*           (x,y) coordinates one per line.                                 */
/*                                                                           */
/* Author:   John Gauch                                                      */
/*                                                                           */
/* Date:     April 29, 1994                                                  */
/*                                                                           */
/* Note:     Copyright (C) The University of Kansas, 1994                    */
/*---------------------------------------------------------------------------*/
#include <IM.h>

/*---------------------------------------------------------------------------*/
/* Purpose:  This is the main program.                                       */
/*---------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
   /* Image variables */
   char Name1[50];
   char Name2[50];
   char Name3[50];
   IM_TYPE *Image1;
   IM_TYPE *Image2;
   FLOAT_TYPE **Data1;
   FLOAT_TYPE **Data2;
   int PixType, Xdim, Ydim, Zdim, DimCnt;

   /* Program variables */
   int Debug = FALSE;
   float Intensity = 255;
   float fx, fy;
   int i = 0, x, y;
   FILE *fd;

   /* Interpret program options */
   printf("PLOT_PTS Program - KUIM Version 1.0\n\n");
   while ((++i < argc) && (argv[i][0] == '-'))
      switch (argv[i][1])
      {
      case 'i':
	 if (sscanf(argv[++i], "%f", &Intensity) == 0)
	    Error("Could not get integer argument");
	 break;
      case 'd':
	 Debug = TRUE;
	 break;
      default:
	 Error("Invalid option encountered");
	 break;
      }

   /* Check number of file names */
   if (argc - i != 3)
   {
      fprintf(stderr, "Usage: plot_pts [options] ptfile infile outfile\n");
      fprintf(stderr, "       [-d]    Print debugging information\n");
      fprintf(stderr, "       [-i #]  Intensity of plotted points (255)\n");
      exit(1);
   }

   /* Get image file names from argument list */
   if (sscanf(argv[i++], "%s", Name3) == 0)
      Error("Could not get input file name");
   if (sscanf(argv[i++], "%s", Name1) == 0)
      Error("Could not get input file name");
   if (sscanf(argv[i++], "%s", Name2) == 0)
      Error("Could not get output file name");

   /* Open point file */
   if ((fd = fopen(Name3, "r")) == NULL)
      Error("Could not open point file");

   /* Read input image */
   Image1 = im_open(Name1, &PixType, &Xdim, &Ydim, &Zdim, &DimCnt);
   if (DimCnt != 2)
      Error("Can not process 1D or 3D images");
   Data1 = (FLOAT_TYPE **) im_alloc2D(Image1, FLOAT);
   im_read(Image1, FLOAT, (char *) &(Data1[0][0]));

   /* Create output image */
   Image2 = im_create(Name2, PixType, Xdim, Ydim, Zdim);
   Data2 = (FLOAT_TYPE **) im_alloc2D(Image2, FLOAT);

   /* Copy original pixel values */
   for (y = 0; y < Ydim; y++)
      for (x = 0; x < Xdim; x++)
	 Data2[y][x] = Data1[y][x];

   /* Plot points read from data file */
   while (fscanf(fd, "%f %f\n", &fx, &fy) != EOF)
   {
      x = (int) (0.5 + fx);
      y = (int) (0.5 + fy);
      if (x < 0)
	 x = 0;
      if (y < 0)
	 y = 0;
      if (x > Xdim - 2)
	 x = Xdim - 2;
      if (y > Ydim - 2)
	 y = Ydim - 2;
      Data2[y][x] = Intensity;
      Data2[y][x + 1] = Intensity;
      Data2[y + 1][x] = Intensity;
      Data2[y + 1][x + 1] = Intensity;
   }

   /* Write information to output image */
   im_write(Image2, FLOAT, (char *) &(Data2[0][0]));
   im_free2D((char **) Data1);
   im_free2D((char **) Data2);
   fclose(fd);
   return (0);
}
