/*---------------------------------------------------------------------------*/
/* Program:  sphere.c                                                        */
/*                                                                           */
/* Purpose:  This program generates a range image of a 3D sphere centered    */
/*           at the center of the image.                                     */
/*                                                                           */
/* Author:   John Gauch                                                      */
/*                                                                           */
/* Date:     March 30, 1994                                                  */
/*                                                                           */
/* Note:     Copyright (C) The University of Kansas, 1994                    */
/*---------------------------------------------------------------------------*/
#include <IM.h>

/*---------------------------------------------------------------------------*/
/* Purpose:  This is the main program.                                       */
/*---------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
   /* Image variables */
   char Name1[50];
   IM_TYPE *Image1;
   FLOAT_TYPE **Data1;
   int Xdim = 256, Ydim = 256, Zdim = 1;

   /* Program variables */
   int Debug = FALSE;
   float Radius = 100;
   float temp;
   int i = 0, x, y;

   /* Interpret program options */
   printf("SPHERE Program - KUIM Version 1.0\n\n");
   while ((++i < argc) && (argv[i][0] == '-'))
      switch (argv[i][1])
      {
      case 'r':
	 if (sscanf(argv[++i], "%f", &Radius) == 0)
	    Error("Could not get integer argument");
	 if (Radius < 0)
	    Error("Radius must be positive");
	 break;
      case 'd':
	 Debug = TRUE;
	 break;
      default:
	 Error("Invalid option encountered");
	 break;
      }

   /* Check number of file names */
   if (argc - i != 1)
   {
      fprintf(stderr, "Usage: sphere [options] outfile\n");
      fprintf(stderr, "       [-d]    Print debugging information\n");
      fprintf(stderr, "       [-r #]  Radius of sphere\n");
      exit(1);
   }

   /* Get image file names from argument list */
   if (sscanf(argv[i++], "%s", Name1) == 0)
      Error("Could not get input file name");

   /* Create output image */
   Image1 = im_create(Name1, FLOAT, Xdim, Ydim, Zdim);
   Data1 = (FLOAT_TYPE **) im_alloc2D(Image1, FLOAT);

   /* Generate output image */
   for (y = 0; y < Ydim; y++)
      for (x = 0; x < Xdim; x++)
      {
	 temp = Radius * Radius - (Xdim / 2 - x) * (Xdim / 2 - x) - (Ydim / 2 - y) * (Ydim / 2 - y);
	 if (temp > 0)
	    Data1[y][x] = (FLOAT_TYPE) sqrt((double) temp);
	 else
	    Data1[y][x] = 0.0;
      }

   /* Write information to output image */
   im_write(Image1, FLOAT, (char *) &(Data1[0][0]));
   im_free2D((char **) Data1);
   return (0);
}
