/*---------------------------------------------------------------------------*/
/* Program:  title.c                                                         */
/*                                                                           */
/* Purpose:  This program updates the title field of a KUIM format image.    */
/*                                                                           */
/* Author:   John Gauch                                                      */
/*                                                                           */
/* Date:     May 4, 1994                                                     */
/*                                                                           */
/* Note:     Copyright (C) The University of Kansas, 1994                    */
/*---------------------------------------------------------------------------*/
#include <IM.h>

/*---------------------------------------------------------------------------*/
/* Purpose:  This is the main program.                                       */
/*---------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
   /* Image variables */
   char Name1[50];
   IM_TYPE *Image1;
   FLOAT_TYPE *Data1;
   int PixType, Xdim, Ydim, Zdim, DimCnt;

   /* Program variables */
   int Debug = FALSE;
   int NewTitle = FALSE;
   int i = 0;
   char Title[nTITLE];
   char OldTitle[nTITLE];
   char Command[50];

   /* Interpret program options */
   fprintf(stderr, "TITLE Program - KUIM Version 1.0\n\n");
   while ((++i < argc) && (argv[i][0] == '-'))
      switch (argv[i][1])
      {
      case 't':
	 NewTitle = TRUE;
	 strcpy(Title, argv[++i]);
	 break;
      case 'd':
	 Debug = TRUE;
	 break;
      default:
	 Error("Invalid option encountered");
	 break;
      }

   /* Check number of file names */
   if (argc - i != 1)
   {
      fprintf(stderr, "Usage: title [options] imagefile\n");
      fprintf(stderr, "       [-d]  Print debugging information\n");
      fprintf(stderr, "       [-t new-title]  Title string\n");
      exit(1);
   }

   /* Get image file names from argument list */
   if (sscanf(argv[i++], "%s", Name1) == 0)
      Error("Could not get input file name");

   /* Open input image */
   Image1 = im_open(Name1, &PixType, &Xdim, &Ydim, &Zdim, &DimCnt);

   /* Print title information */
   im_get_title(Image1, OldTitle);
   if (NewTitle == FALSE)
   {
      printf("%s\n", OldTitle);
      exit(0);
   }
   else
   {
      fprintf(stderr, "Old title '%s'\n", OldTitle);
      fprintf(stderr, "New title '%s'\n", Title);
   }

   /* Read input image */
   Data1 = (FLOAT_TYPE *) im_alloc1D(Image1, FLOAT);
   im_read(Image1, FLOAT, (char *) Data1);

   /* Remove input image */
   sprintf(Command, "rm %s", Name1);
   system(Command);

   /* Save title in new output image */
   Image1 = im_create(Name1, FLOAT, Xdim, Ydim, Zdim);
   im_put_title(Image1, Title);
   im_write(Image1, FLOAT, (char *) Data1);
   im_free1D((char *) Data1);
   return (0);
}
