/*---------------------------------------------------------------------------*/
/* Program:  xdisp.c                                                         */
/*                                                                           */
/* Purpose:  This program displays KUIM format images in an X window.        */
/*                                                                           */
/* Author:   John Gauch                                                      */
/*                                                                           */
/* Date:     March 28, 1994 - Version 1.                                     */
/*           July 24, 1994 - Version 2.                                      */
/*           August 3, 1994 - Version 3.                                     */
/*           October 7, 1994 - Version 4.                                    */
/*           April 6, 1995 - Version 5 - display of selected points          */
/*                                                                           */
/* Note:     NEED TO ADD JPEG_RGB BACK IN AGAIN...                           */
/* Note:     Copyright (C) The University of Kansas, 1994-1995               */
/*---------------------------------------------------------------------------*/
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <IM.h>

/* Global variables */
unsigned char Red[nCMAP];
unsigned char Green[nCMAP];
unsigned char Blue[nCMAP];
int PixType;
int MarkPoint = FALSE;

/* Function prototypes */
void Display1D(char *Name, BYTE_TYPE * Data, int Xdim);
void Display2D(char *Name, BYTE_TYPE * Data, int Xdim, int Ydim);
void Display3D(char *Name, BYTE_TYPE * Data, int Xdim, int Ydim, int Zdim);

/*---------------------------------------------------------------------------*/
/* Purpose:  This routine displays a 1D image.                               */
/*---------------------------------------------------------------------------*/
void Display1D(char *Name, BYTE_TYPE * Data, int Xdim)
{
   IM_TYPE *Image2;
   BYTE_TYPE **Data2;
   int x, y, Ydim = 256;

   /* Create 2D plot of 1D image */
   Image2 = im_create("/dev/null", BYTE, Xdim, Ydim, 1);
   Data2 = (BYTE_TYPE **) im_alloc2D(Image2, BYTE);
   for (y = 0; y < Ydim; y++)
      for (x = 0; x < Xdim; x++)
	 if ((Ydim - y) < (Data[x] - 50))
	    Data2[y][x] = 250;
	 else
	    Data2[y][x] = 50;
   Display2D(Name, &(Data2[0][0]), Xdim, Ydim);
}

/*---------------------------------------------------------------------------*/
/* Purpose: This routine displays a 2D image.                                */
/*---------------------------------------------------------------------------*/
void Display2D(char *Name, BYTE_TYPE * Data, int Xdim, int Ydim)
{
   Display *display;
   char *display_name = NULL;
   int screen_num;
   int depth;
   unsigned long foreground;
   unsigned long background;
   Window window;
   GC gc;
   Visual *visual;
   Colormap old_colormap;
   Colormap new_colormap;
   int i;
   XColor color;
   double Gamma = 0.01;
   XImage *ximage;
   int done = FALSE;
   XEvent event;
   char text[50];
   KeySym key;

   /* Set up window for display */
   if ((display = XOpenDisplay(display_name)) == NULL)
      Error("Can not connect to X server");
   screen_num = DefaultScreen(display);
   depth = DefaultDepth(display, screen_num);
   if (depth != 8)
      Error("Can only display on 8 bit displays");
   background = BlackPixel(display, screen_num);
   foreground = WhitePixel(display, screen_num);
   window = XCreateSimpleWindow(display, DefaultRootWindow(display),
			   200, 200, Xdim, Ydim, 5, foreground, background);
   XStoreName(display, window, Name);
   XSelectInput(display, window, ButtonPressMask | KeyPressMask | ExposureMask);
   gc = XCreateGC(display, window, 0, 0);
   XSetBackground(display, gc, background);
   XSetForeground(display, gc, foreground);
   visual = XDefaultVisualOfScreen(DefaultScreenOfDisplay(display));
   XMapRaised(display, window);

   /* Use image colormap */
   if ((PixType == PSEUDO) || (PixType == COLOR) || (PixType == JPEG_RGB))
   {
      new_colormap = XCreateColormap(display, window, visual, AllocAll);
      for (i = 0; i <= 255; i++)
      {
	 color.pixel = i;
	 color.flags = DoRed | DoGreen | DoBlue;
	 color.red = 256 * Red[i];
	 color.blue = 256 * Blue[i];
	 color.green = 256 * Green[i];
	 XStoreColor(display, new_colormap, &color);
      }
   }

   /* Define new colormap */
   else
   {
      old_colormap = DefaultColormap(display, screen_num);
      new_colormap = XCreateColormap(display, window, visual, AllocAll);
      for (i = 0; i <= 255; i++)
      {
	 color.pixel = i;
	 XQueryColor(display, old_colormap, &color);
	 color.flags = DoRed | DoGreen | DoBlue;
	 XStoreColor(display, new_colormap, &color);
      }
      for (i = 50; i <= 250; i++)
      {
	 color.pixel = i;
	 color.red = color.blue = color.green = (int) (65535
				    * log((double) (1.0 + Gamma * (i - 50)))
				       / log((double) (1.0 + Gamma * 200)));
	 XStoreColor(display, new_colormap, &color);
      }
      /* Add RED entry for marking points */
      color.pixel = 251;
      color.green = 0;
      color.red = 255 * 255;
      color.blue = 0;
      XStoreColor(display, new_colormap, &color);
   }

   /* Copy image to display window */
   XSetWindowColormap(display, window, new_colormap);
   ximage = XCreateImage(display, visual, 8, ZPixmap, 0, Data, Xdim, Ydim, 8, 0);

   /* Loop handling events */
   while (done == FALSE)
   {
      XNextEvent(display, &event);
      switch (event.type)
      {
      case Expose:
	 if (event.xexpose.count == 0)
	    XPutImage(display, window, gc, ximage, 0, 0, 0, 0, Xdim, Ydim);
	 break;
      case MappingNotify:
	 XRefreshKeyboardMapping((XMappingEvent *) & event);
	 break;
      case ButtonPress:
	 printf("%d %d\n", event.xbutton.x, event.xbutton.y);
	 if (MarkPoint == TRUE)
	 {
	    Data[event.xbutton.y * Xdim + event.xbutton.x] = 251;
	    Data[event.xbutton.y * Xdim + event.xbutton.x + 1] = 251;
	    Data[event.xbutton.y * Xdim + event.xbutton.x + Xdim] = 251;
	    Data[event.xbutton.y * Xdim + event.xbutton.x + Xdim + 1] = 251;
	    XPutImage(display, window, gc, ximage, 0, 0, 0, 0, Xdim, Ydim);
	 }
	 break;
      case KeyPress:
	 /* Exit if 'q' was pressed */
	 i = XLookupString((XKeyEvent *) & event, text, 50, &key, 0);
	 if (i == 1 && text[0] == 'q')
	    done = TRUE;
	 break;
      }
   }
}

/*---------------------------------------------------------------------------*/
/* Purpose: This routine displays a 3D image.                                */
/*---------------------------------------------------------------------------*/
void Display3D(char *Name, BYTE_TYPE * Data, int Xdim, int Ydim, int Zdim)
{
   Display *display;
   char *display_name = NULL;
   int screen_num;
   int depth;
   unsigned long foreground;
   unsigned long background;
   Window window;
   GC gc;
   Visual *visual;
   Colormap old_colormap;
   Colormap new_colormap;
   int i;
   XColor color;
   double Gamma = 0.01;
   XImage *ximage;
   int done = FALSE;
   XEvent event;
   char text[50];
   KeySym key;
   int offset;
   int slice = 0;
   int pixcnt;

   /* Set up window for display */
   if ((display = XOpenDisplay(display_name)) == NULL)
      Error("Can not connect to X server");
   screen_num = DefaultScreen(display);
   depth = DefaultDepth(display, screen_num);
   if (depth != 8)
      Error("Can only display on 8 bit displays");
   background = BlackPixel(display, screen_num);
   foreground = WhitePixel(display, screen_num);
   window = XCreateSimpleWindow(display, DefaultRootWindow(display),
			   200, 200, Xdim, Ydim, 5, foreground, background);
   XStoreName(display, window, Name);
   XSelectInput(display, window, ButtonPressMask | KeyPressMask | ExposureMask);
   gc = XCreateGC(display, window, 0, 0);
   XSetBackground(display, gc, background);
   XSetForeground(display, gc, foreground);
   visual = XDefaultVisualOfScreen(DefaultScreenOfDisplay(display));
   XMapRaised(display, window);

   /* Use image colormap */
   if ((PixType == PSEUDO) || (PixType == COLOR) || (PixType == JPEG_RGB))
   {
      new_colormap = XCreateColormap(display, window, visual, AllocAll);
      for (i = 0; i <= 255; i++)
      {
	 color.pixel = i;
	 color.flags = DoRed | DoGreen | DoBlue;
	 color.red = 256 * Red[i];
	 color.blue = 256 * Blue[i];
	 color.green = 256 * Green[i];
	 XStoreColor(display, new_colormap, &color);
      }
   }

   /* Define new colormap */
   else
   {
      old_colormap = DefaultColormap(display, screen_num);
      new_colormap = XCreateColormap(display, window, visual, AllocAll);
      for (i = 0; i <= 255; i++)
      {
	 color.pixel = i;
	 XQueryColor(display, old_colormap, &color);
	 color.flags = DoRed | DoGreen | DoBlue;
	 XStoreColor(display, new_colormap, &color);
      }
      for (i = 50; i <= 250; i++)
      {
	 color.pixel = i;
	 color.blue = color.green = color.red = (int) (65535
				    * log((double) (1.0 + Gamma * (i - 50)))
				       / log((double) (1.0 + Gamma * 200)));
	 XStoreColor(display, new_colormap, &color);
      }
   }

   /* Copy first slice to display window */
   XSetWindowColormap(display, window, new_colormap);
   ximage = XCreateImage(display, visual, 8, ZPixmap, 0, Data, Xdim, Ydim, 8, 0);
   pixcnt = Xdim * Ydim;

   /* Loop handling events */
   while (done == FALSE)
   {
      XNextEvent(display, &event);
      switch (event.type)
      {
      case Expose:
	 /* Draw the window only if this is the last contiguous response */
	 if (event.xexpose.count == 0)
	    XPutImage(display, window, gc, ximage, 0, 0, 0, 0, Xdim, Ydim);
	 break;
      case MappingNotify:
	 /* Mappings have changed, refresh them */
	 XRefreshKeyboardMapping((XMappingEvent *) & event);
	 break;
      case ButtonPress:
	 /* Display movie forwards */
	 if (event.xbutton.button == 3)
	 {
	    for (slice = 0; slice < Zdim; slice++)
	    {
	       offset = pixcnt * slice;
	       ximage = XCreateImage(display, visual, 8, ZPixmap, 0,
				     &(Data[offset]), Xdim, Ydim, 8, 0);
	       XPutImage(display, window, gc, ximage, 0, 0, 0, 0, Xdim, Ydim);
	       sprintf(text, "slice %d", slice);
	       XStoreName(display, window, text);
	    }
	    slice = Zdim - 1;
	 }

	 /* Single step forward through movie */
	 if (event.xbutton.button == 2)
	 {
	    slice = (slice + 1) % Zdim;
	    offset = pixcnt * slice;
	    ximage = XCreateImage(display, visual, 8, ZPixmap, 0,
				  &(Data[offset]), Xdim, Ydim, 8, 0);
	    XPutImage(display, window, gc, ximage, 0, 0, 0, 0, Xdim, Ydim);
	    sprintf(text, "slice %d", slice);
	    XStoreName(display, window, text);
	 }

	 /* Display movie backwards */
	 if (event.xbutton.button == 1)
	 {
	    pixcnt = Xdim * Ydim;
	    for (slice = (Zdim - 1); slice >= 0; slice--)
	    {
	       offset = pixcnt * slice;
	       ximage = XCreateImage(display, visual, 8, ZPixmap, 0,
				     &(Data[offset]), Xdim, Ydim, 8, 0);
	       XPutImage(display, window, gc, ximage, 0, 0, 0, 0, Xdim, Ydim);
	       sprintf(text, "slice %d", slice);
	       XStoreName(display, window, text);
	    }
	    slice = 0;
	 }
	 break;
      case KeyPress:
	 /* Exit if 'q' was pressed */
	 i = XLookupString((XKeyEvent *) & event, text, 50, &key, 0);
	 if (i == 1 && text[0] == 'q')
	    done = TRUE;
	 break;
      }
   }
}

/*---------------------------------------------------------------------------*/
/* Purpose:  This is the main program.                                       */
/*---------------------------------------------------------------------------*/
main(int argc, char *argv[])
{
   /* Image variables */
   char Name1[50];
   IM_TYPE *Image1;
   IM_TYPE *Image2;
   BYTE_TYPE *Data1;
   BYTE_TYPE **Data2;
   int Xdim, Ydim, Zdim, DimCnt;

   /* Program variables */
   int Count, Min, Max;
   int Debug = FALSE;
   int Unfold = FALSE;
   int Rows, Cols;
   int NewXdim, NewYdim;
   int i = 0, x, y, z, r, c;

   /* Interpret program options */
   fprintf(stderr, "XDISP Program - KUIM Version 4.0\n\n");
   while ((++i < argc) && (argv[i][0] == '-'))
      switch (argv[i][1])
      {
      case 'u':
	 Unfold = TRUE;
	 break;
      case 'd':
	 Debug = TRUE;
	 break;
      case 'm':
	 MarkPoint = TRUE;
	 break;
      default:
	 Error("Invalid option encountered");
	 break;
      }

   /* Check number of file names */
   if (argc - i != 1)
   {
      fprintf(stderr, "Usage: xdisp [options] infile\n");
      fprintf(stderr, "       [-d] Print debugging information\n");
      fprintf(stderr, "       [-u] Unfold 3D images\n");
      fprintf(stderr, "       [-m] Mark selected points in red\n");
      exit(1);
   }

   /* Get image file names from argument list */
   if (sscanf(argv[i++], "%s", Name1) == 0)
      Error("Could not get input file name");

   /* Read color input image */
   Image1 = im_open(Name1, &PixType, &Xdim, &Ydim, &Zdim, &DimCnt);
   Data1 = (BYTE_TYPE *) im_alloc1D(Image1, BYTE);
   if ((PixType == PSEUDO) || (PixType == COLOR) || (PixType == JPEG_RGB))
   {
      im_read(Image1, PSEUDO, (char *) Data1);
      im_get_cmap(Image1, Red, Green, Blue);
   }

   /* Read grey input image */
   else
   {
      im_read(Image1, BYTE, (char *) Data1);
      Count = Xdim * Ydim * Zdim;
      Min = Max = Data1[0];
      for (i = 0; i < Count; i++)
	 if (Min > Data1[i])
	    Min = Data1[i];
	 else if (Max < Data1[i])
	    Max = Data1[i];
      for (i = 0; i < Count; i++)
	 if (Max > Min)
	    Data1[i] = 50 + ((Data1[i] - Min) * 200) / (Max - Min);
	 else
	    Data1[i] = 50;
   }

   /* Display image */
   if (DimCnt == 1)
      Display1D(Name1, Data1, Xdim);
   if (DimCnt == 2)
      Display2D(Name1, Data1, Xdim, Ydim);
   else if ((DimCnt == 3) && (Unfold == FALSE))
      Display3D(Name1, Data1, Xdim, Ydim, Zdim);
   else if ((DimCnt == 3) && (Unfold == TRUE))
   {
      /* Allocate unfolded image */
      Cols = (int) ceil(sqrt((double) Zdim));
      Rows = (int) ceil((double) Zdim / (double) Cols);
      NewXdim = Xdim * Cols;
      NewYdim = Ydim * Rows;
      Image2 = im_create("/dev/null", BYTE, NewXdim, NewYdim, 1);
      Data2 = (BYTE_TYPE **) im_alloc2D(Image2, BYTE);

      /* Initialize output image */
      for (y = 0; y < NewYdim; y++)
	 for (x = 0; x < NewXdim; x++)
	    Data2[y][x] = 50;

      /* Unfold image along both directions */
      for (r = 0; r < Rows; r++)
	 for (c = 0; c < Cols; c++)
	 {
	    z = r * Cols + c;
	    if (z < Zdim)
	       for (y = 0; y < Ydim; y++)
		  for (x = 0; x < Xdim; x++)
		     Data2[y + r * Ydim][x + c * Xdim] = Data1[z * Xdim * Ydim + y * Xdim + x];
	    for (x = 0; x < Xdim; x++)
	       Data2[r * Ydim][x + c * Xdim] = 128;
	    for (y = 0; y < Ydim; y++)
	       Data2[y + r * Ydim][c * Xdim] = 128;
	 }
      Display2D(Name1, &(Data2[0][0]), NewXdim, NewYdim);
      im_free2D((char **) Data2);
   }

   im_free1D((char *) Data1);
   exit(0);
}
